#ifndef __PSQUAREQUANTILE_H__
#define __PSQUAREQUANTILE_H__

#include <mathematics.h>

//!Approximation des p-Quantils nach P-Methode.
/*!
 * Diese Klasse realisiert die Approximation von Quantilen aus Simulationsdaten
 * ohne Verwaltung aller Messwerte.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu.berlin.de>
 */
class PSquareQuantile
{ 
friend void QuantileTest(double);
public:
	
	//! Konstruktor.
	PSquareQuantile(double p=0.5);

	//! Destruktur.
	~PSquareQuantile() {};

	//! Approximation mit neuem Wert aktualisieren.
	void advance(double value);

	//! Zuruecksetzen.
	void reset();

	//! Approximierter Wert fuer p-Quantil.
	inline double quantile() const
	{
		return _q[2];
	}
	
	//! Dichte fuer akuell approximiertes p-Quantil.
	inline double density() const
	{
		double a[3];
		
		// q_i = a0*p_i^2 + a1*p_i + a2 = F^-1(y)
		parabola(_p, _q+1, a);
		
		// f(xp) = (a1 + 2*a0*F(xp))^-1 = (a1 + 2*a0*p)^-1
		return (1.0 / (a[1] + 2.0*a[0]*_p[1]));
	}
	
	//! Minimaler Wert.
	inline double min() const
	{
		return _q[0];
	}

	//! Maximaler Wert.
	inline double max() const
	{
		return _q[4];
	}
	
protected:
	
	//! p/2, p, (1+p)/2
	double _p[3];
	
	//! Hoehen der Marker.
	double _q[5];

	//! Positionen der Marker.
	double _n[5];

	//! Neue Positionen der Marker.
	double _nn[5];

	//! Inkrement fuer Positionen der Marker.
	double _dn[5];

	//! Zaehler fuer Initialisierungsphase.
	unsigned int _init;
};

#endif
