#ifndef __FACTORY_H
#define __FACTORY_H

/* Alternative approach at implementing a parametrized factory function by
 * employing compile-time registration of factory methods via templates.
 *
 * Compile with g++ -O2 -Wall -o tagfactory tagfactory.cc
 *
 * (c)2002, 2003 Frerich Raabe <raabe@kde.org>
 */

#include <iostream>
#include <map>
#include <string>

class FactoryElement
{
	public:
		typedef FactoryElement *( *FactoryFn )();
		
		class Factory
		{
			public:
				static Factory &self()
				{
					if ( !m_instance )
						m_instance = new Factory;
					return *m_instance;
				}

				void announce( FactoryElement::FactoryFn factoryFn,
				              const std::string &tag )
				{
					if ( m_elementMap.find( tag ) == m_elementMap.end() ) {
						m_elementMap[tag] = factoryFn;
					}
				}

				FactoryElement *create( const std::string &tag )
				{
					ElementMap::const_iterator it = m_elementMap.find( tag );
					if ( it != m_elementMap.end() )
						return it->second();
					return 0;
				}

			private:
				Factory()  {}
				Factory( const Factory &rhs );
				Factory &operator=( const Factory &rhs );

				static Factory *m_instance;

				typedef std::map<std::string, FactoryElement::FactoryFn> ElementMap;
				ElementMap m_elementMap;
		};
		
		virtual ~FactoryElement() {};

};

template <class T>
class Registrar
{
	public:
		Registrar( const std::string &tag )
		{
			FactoryElement::Factory::self().announce( &factoryFn, tag );
		}

		static FactoryElement *factoryFn()
		{
			return new T;
		}
};

#define REGISTER_ELEMENT(Class, Tag) \
Registrar<Class > Class##Registrar(Tag);

#endif
