#include <string.h>
#include <vector>
#include "compound_iterator.h"
#if !defined(MATRIX_)
#define MATRIX_H

#include <iostream>
#include <stdlib.h>

template <class T>
class Matrix
{
public:
	typedef std::vector<T> row_type; // row
        typedef std::vector<row_type> matrix; // matrix
	
	typedef T				value_type;
	typedef value_type*			pointer;
	typedef std::vector<T>			vector_type;
	typedef __gnu_cxx::__normal_iterator<pointer, vector_type> iterator;
        
	typedef compound_iterator<T, typename matrix::iterator> col_iterator;
        typedef compound_iterator<T, typename matrix::reverse_iterator> reverse_col_iterator;
	typedef const compound_iterator<T, typename matrix::iterator> const_col_iterator;
        typedef const compound_iterator<T, typename matrix::reverse_iterator> const_reverse_col_iterator;

	typedef __gnu_cxx::__normal_iterator<pointer, vector_type> row_iterator;
	typedef const row_iterator const_row_iterator; 
	
protected:

	int m_nXSize;
        int m_nYSize;

	// inner matrix representation
	matrix* m_Matrix;

public:
        //! Constructor 
	Matrix(int nYSize, int nXSize);
        //! Copy Constructor
	Matrix(const Matrix& matrix);

        Matrix& operator= (const Matrix& matrix);
        Matrix  operator+ (const T& item);
        Matrix  operator- (const T& item);

	std::vector<T>& operator[] (int) const;
	
	//! Returns the sum of the matrix elements
        T SumAll();
	
        //! Returns the X-Size of the matrix
        int getXSize();
	//! Returns the Y-Size of the matrix
        int getYSize();
        
	//! Gets the min matrix-value (Element Type needs < operator definition)
	T getMinValue();
	//! Gets the min matrix-value (Element Type needs < operator dedinition)
        T getMaxValue();

	const_col_iterator const_col(int x) { return const_col_iterator(m_Matrix->iterator, x); };
        col_iterator col(int x) { return col_iterator(m_Matrix->iterator, x); };

	const_col_iterator const_col_begin(int x) { return const_col_iterator(m_Matrix->begin(), x); };
	const_col_iterator const_col_end(int x) { return const_col_iterator(m_Matrix->end(), x); };

	col_iterator col_begin(int x) { return col_iterator(m_Matrix->begin(), x); };
	col_iterator col_end(int x) { return col_iterator(m_Matrix->end(), x); };

	row_iterator row(int y) { return m_Matrix->at(y).begin(); }
	row_iterator row_begin(int y) { return m_Matrix->at(y).begin(); }
	row_iterator row_end(int y) { return m_Matrix->at(y).end(); }
	
	iterator begin() {
		return m_Matrix->begin();
	}

	iterator end() {
		return m_Matrix->end();
	}

	
	matrix& innerMatrix() { return *m_Matrix; }; 
	
	virtual ~Matrix();
};


template<class T>
Matrix<T>::Matrix(int nYSize, int nXSize)
{
	m_Matrix = new matrix(nYSize, row_type(nXSize));

	m_nXSize = nXSize;
        m_nYSize = nYSize;

}

template<class T>
Matrix<T>::Matrix(const Matrix& matrix)
{
	std::cerr << "Not implemented yet" << std::endl;
	exit(1);
}

template<class T>
std::vector<T>& Matrix<T>::operator[] (int i) const {
	return (*m_Matrix)[i];
}



template<class T>
Matrix<T>& Matrix<T>::operator= (const Matrix& matrix)
{
	std::cerr << "Not implemented yet" << std::endl;
	exit(1);
}

template<class T>
T Matrix<T>::getMinValue()
{
        T minValue = (*m_Matrix)[0][0];
        int i,j;

        for (i=0; i<m_nYSize; i++)
                for (j=0; j<m_nXSize; j++) {
                        if((*m_Matrix)[i][j]<minValue)
                                minValue = (*m_Matrix)[i][j];
                }
        return minValue;
}

template<class T>
T Matrix<T>::getMaxValue()
{
        T maxValue = m_Matrix[0][0];
        int i,j;

        for (i=0; i<m_nYSize; i++)
                for (j=0; j<m_nXSize; j++) {
                        if(m_Matrix[i][j]>maxValue)
                                maxValue = (*m_Matrix)[i][j];
                }
        return maxValue;
}

template<class T>
Matrix<T> Matrix<T>::operator+ (const T& item)
{
        int i, j;

        Matrix<T> mtrx(m_nYSize, m_nXSize);
        for (i=0; i<m_nYSize; i++)
                for (j=0; j<m_nXSize; j++) {
                        (*mtrx.m_Matrix)[i][j] = (*m_Matrix)[i][j]+item ;
                }
        return mtrx;
}

template<class T>
Matrix<T> Matrix<T>::operator- (const T& item)
{
        int i, j;

        Matrix<T> mtrx(m_nYSize, m_nXSize);
        for (i=0; i<m_nYSize; i++)
                for (j=0; j<m_nXSize; j++) {
                        (*mtrx.m_Matrix)[i][j] = (*m_Matrix)[i][j]-item ;
                }
        return mtrx;
}

template<class T>
Matrix<T>::~Matrix()
{
	//delete m_Matrix;
}

template<class T>
int Matrix<T>::getXSize()
{
        return m_nXSize;
}

template<class T>
T Matrix<T>::SumAll()
{
        T sum = 0;
        int i, j;

        for (i=0; i<m_nYSize; i++)
                for (j=0; j<m_nXSize; j++) {
                        sum += (*m_Matrix)[i][j];
                }
        return sum;
}

template<class T>
int Matrix<T>::getYSize()
{
        return m_nYSize;
}

#endif // !defined(MATRIX_H)
