#ifndef __WIREMATRIXMODIFIER__
#define __WIREMATRIXMODIFIER__

#include <list>

class WireMatrixIterator;
class WireMatrixModifier;

typedef std::list<WireMatrixModifier*> ModifierList;

//! Interface fuer Schaltmatrix-Manipulatoren. 
/*!
 * Ein Schaltmatrix-Manipulator veraendert die Eintraege einer 
 * Schaltmatrix eines Routers. Zusammen mit einem Iterator, kann 
 * eine ganze Zeile oder Spalte modifiziert werden.
 * 
 * @see WireMatrixIterator
 * @see WireMatrixEntry
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class WireMatrixModifier
{
public:
	//! Rueckgabewerte fuer Test der Vorbedingung.
	enum e_precond_return 
	{
		FAILED,		//Pre-Condition verletzt (false)
		PASSED,		//Pre-Condition erfuellt (true)
		SKIPPED		//Keine abschliessende Entscheidung
	};

	//! Vorbedingung.
	/*!
	 * Bevor eine Modifikation eines Eintrages einer Schaltmatrix
	 * erfolgt, muessen u.U. verschiedene Randbedingungen erfuellt sein.
	 * Der Test dieser Randbedingungen soll in dieser Methode formuliert 
	 * werden.
	 *
	 * @param it Iterator zu Eintrag einer Schaltmatrix.
	 * @return Status (e_precond_return).
	 */
	virtual int precondition(WireMatrixIterator* it) const { return PASSED; }
	
	//! Modifikation.
	/*!
	 * Die Modifikation eines Eintrages einer Schaltmatrix wird in
	 * dieser Methode implementiert.
	 *
	 * @param it Iterator zu Eintrag einer Schaltmatrix.
	 */
	virtual void modify(WireMatrixIterator* it) = 0;

	virtual ~WireMatrixModifier() {};
};

#endif

